@@ -19,12 +19,7 @@ The geodesic distance is the shortest distance on the surface of an
 ellipsoidal model of the earth.  The default algorithm uses the method
 is given by `Karney (2013)
 <https://doi.org/10.1007%2Fs00190-012-0578-z>`_ (:class:`.geodesic`);
-this is accurate to round-off and always converges.  An older
-*deprecated* method due to `Vincenty (1975)
-<https://en.wikipedia.org/wiki/Vincenty's_formulae>`_
-(:class:`.vincenty`) is also available; this is only accurate to 0.2 mm
-and the distance calculation fails to converge for nearly antipodal
-points.
+this is accurate to round-off and always converges.
 
 ``geopy.distance.distance`` currently uses :class:`.geodesic`.
 
@@ -43,7 +38,8 @@ most globally accurate.  geopy includes a few other models in the
                   'GRS-67':        (6378.1600,   6356.774719,   1 / 298.25),
                   }
 
-Here are examples of ``distance.distance`` usage::
+Here are examples of ``distance.distance`` usage, taking pair
+of :code:`(lat, lon)` tuples::
 
     >>> from geopy import distance
     >>> newport_ri = (41.49008, -71.312796)
@@ -56,8 +52,6 @@ Here are examples of ``distance.distance`` usage::
     >>> print(distance.distance(wellington, salamanca).km)
     19959.6792674
 
-The second example above fails with :class:`.vincenty`.
-
 Using :class:`.great_circle` distance::
 
     >>> print(distance.great_circle(newport_ri, cleveland_oh).miles)
@@ -85,16 +79,48 @@ calculate the length of a path::
     >>> print((d(ne, cl) + d(cl, wa) + d(wa, pa)).miles)
     3277.30439191
 
-"""
-from __future__ import division
 
-import warnings
-from math import asin, atan, atan2, cos, pi, sin, sqrt, tan
+.. _distance_altitudes:
+
+Currently all algorithms assume that altitudes of the points are either
+zero (as in the examples above) or equal, and are relatively small.
+Thus altitudes never affect the resulting distances::
+
+    >>> from geopy import distance
+    >>> newport_ri = (41.49008, -71.312796)
+    >>> cleveland_oh = (41.499498, -81.695391)
+    >>> print(distance.distance(newport_ri, cleveland_oh).km)
+    866.4554329098687
+    >>> newport_ri = (41.49008, -71.312796, 100)
+    >>> cleveland_oh = (41.499498, -81.695391, 100)
+    >>> print(distance.distance(newport_ri, cleveland_oh).km)
+    866.4554329098687
+
+If you need to calculate distances with elevation, then for short
+distances the `Euclidean distance
+<https://en.wikipedia.org/wiki/Euclidean_distance>`_ formula might give
+a suitable approximation::
+
+    >>> import math
+    >>> from geopy import distance
+    >>> p1 = (43.668613, 40.258916, 0.976)
+    >>> p2 = (43.658852, 40.250839, 1.475)
+    >>> flat_distance = distance.distance(p1[:2], p2[:2]).km
+    >>> print(flat_distance)
+    1.265133525952866
+    >>> euclidian_distance = math.sqrt(flat_distance**2 + (p2[2] - p1[2])**2)
+    >>> print(euclidian_distance)
+    1.359986705262199
+
+An attempt to calculate distances between points with different altitudes
+would result in a :class:`ValueError` exception.
+
+"""
+from math import asin, atan2, cos, sin, sqrt
 
 from geographiclib.geodesic import Geodesic
 
 from geopy import units, util
-from geopy.compat import cmp, py3k, string_compare
 from geopy.point import Point
 from geopy.units import radians
 
@@ -121,6 +147,10 @@ ELLIPSOIDS = {
 }
 
 
+def cmp(a, b):
+    return (a > b) - (a < b)
+
+
 def lonlat(x, y, z=0):
     """
     ``geopy.distance.distance`` accepts coordinates in ``(y, x)``/``(lat, lon)``
@@ -146,13 +176,95 @@ def lonlat(x, y, z=0):
     return Point(y, x, z)
 
 
-class Distance(object):
+def _ensure_same_altitude(a, b):
+    if abs(a.altitude - b.altitude) > 1e-6:
+        raise ValueError(
+            'Calculating distance between points with different altitudes '
+            'is not supported'
+        )
+    # Note: non-zero equal altitudes are fine: assuming that
+    # the elevation is many times smaller than the Earth radius
+    # it won't give much error.
+
+
+class Distance:
     """
-    Base for :class:`.great_circle`, :class:`.vincenty`, and
-    :class:`.geodesic`.
+    Base class for other distance algorithms. Represents a distance.
+
+    Can be used for units conversion::
+
+        >>> from geopy.distance import Distance
+        >>> Distance(miles=10).km
+        16.09344
+
+    Distance instances have all *distance* properties from :mod:`geopy.units`,
+    e.g.: ``km``, ``m``, ``meters``, ``miles`` and so on.
+
+    Distance instances are immutable.
+
+    They support comparison::
+
+        >>> from geopy.distance import Distance
+        >>> Distance(kilometers=2) == Distance(meters=2000)
+        True
+        >>> Distance(kilometers=2) > Distance(miles=1)
+        True
+
+    String representation::
+
+        >>> from geopy.distance import Distance
+        >>> repr(Distance(kilometers=2))
+        'Distance(2.0)'
+        >>> str(Distance(kilometers=2))
+        '2.0 km'
+        >>> repr(Distance(miles=2))
+        'Distance(3.218688)'
+        >>> str(Distance(miles=2))
+        '3.218688 km'
+
+    Arithmetics::
+
+        >>> from geopy.distance import Distance
+        >>> -Distance(miles=2)
+        Distance(-3.218688)
+        >>> Distance(miles=2) + Distance(kilometers=1)
+        Distance(4.218688)
+        >>> Distance(miles=2) - Distance(kilometers=1)
+        Distance(2.218688)
+        >>> Distance(kilometers=6) * 5
+        Distance(30.0)
+        >>> Distance(kilometers=6) / 5
+        Distance(1.2)
     """
 
     def __init__(self, *args, **kwargs):
+        """
+        There are 3 ways to create a distance:
+
+        - From kilometers::
+
+            >>> from geopy.distance import Distance
+            >>> Distance(1.42)
+            Distance(1.42)
+
+        - From units::
+
+            >>> from geopy.distance import Distance
+            >>> Distance(kilometers=1.42)
+            Distance(1.42)
+            >>> Distance(miles=1)
+            Distance(1.609344)
+
+        - From points (for non-abstract distances only),
+          calculated as a sum of distances between all points::
+
+            >>> from geopy.distance import geodesic
+            >>> geodesic((40, 160), (40.1, 160.1))
+            Distance(14.003702498106215)
+            >>> geodesic((40, 160), (40.1, 160.1), (40.2, 160.2))
+            Distance(27.999954644813478)
+        """
+
         kilometers = kwargs.pop('kilometers', 0)
         if len(args) == 1:
             # if we only get one argument we assume
@@ -181,29 +293,75 @@ class Distance(object):
         return self + -other
 
     def __mul__(self, other):
-        return self.__class__(self.kilometers * other)
+        if isinstance(other, Distance):
+            raise TypeError(
+                "Distance instance must be multiplicated with numbers."
+            )
+        else:
+            return self.__class__(self.kilometers * other)
 
-    def __div__(self, other):
+    def __rmul__(self, other):
+        if isinstance(other, Distance):
+            raise TypeError(
+                "Distance instance must be multiplicated with numbers."
+            )
+        else:
+            return self.__class__(other * self.kilometers)
+
+    def __truediv__(self, other):
         if isinstance(other, Distance):
             return self.kilometers / other.kilometers
         else:
             return self.__class__(self.kilometers / other)
 
-    __truediv__ = __div__
+    def __floordiv__(self, other):
+        if isinstance(other, Distance):
+            return self.kilometers // other.kilometers
+        else:
+            return self.__class__(self.kilometers // other)
 
     def __abs__(self):
         return self.__class__(abs(self.kilometers))
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self.kilometers)
 
-    __bool__ = __nonzero__
-
     def measure(self, a, b):
+        # Intentionally not documented, because this method is not supposed
+        # to be used directly.
+        raise NotImplementedError("Distance is an abstract class")
+
+    def destination(self, point, bearing, distance=None):
         """
-        Abstract method for measure
+        Calculate destination point using a starting point, bearing
+        and a distance. This method works for non-abstract distances only.
+
+        Example: a point 10 miles east from ``(34, 148)``::
+
+            >>> import geopy.distance
+            >>> geopy.distance.distance(miles=10).destination((34, 148), bearing=90)
+            Point(33.99987666492774, 148.17419994321995, 0.0)
+
+        :param point: Starting point.
+        :type point: :class:`geopy.point.Point`, list or tuple of ``(latitude,
+            longitude)``, or string as ``"%(latitude)s, %(longitude)s"``.
+
+        :param float bearing: Bearing in degrees: 0 -- North, 90 -- East,
+            180 -- South, 270 or -90 -- West.
+
+        :param distance: Distance, can be used to override
+            this instance::
+
+                >>> from geopy.distance import distance, Distance
+                >>> distance(miles=10).destination((34, 148), bearing=90, \
+distance=Distance(100))
+                Point(33.995238229104764, 149.08238904409637, 0.0)
+
+        :type distance: :class:`.Distance`
+
+        :rtype: :class:`geopy.point.Point`
         """
-        raise NotImplementedError()
+        raise NotImplementedError("Distance is an abstract class")
 
     def __repr__(self):  # pragma: no cover
         return 'Distance(%s)' % self.kilometers
@@ -217,24 +375,34 @@ class Distance(object):
         else:
             return cmp(self.kilometers, other)
 
-    if py3k:
-        def __eq__(self, other):
-            return self.__cmp__(other) == 0
+    def __hash__(self):
+        return hash(self.kilometers)
 
-        def __ne__(self, other):
-            return self.__cmp__(other) != 0
+    def __eq__(self, other):
+        return self.__cmp__(other) == 0
 
-        def __gt__(self, other):
-            return self.__cmp__(other) > 0
+    def __ne__(self, other):
+        return self.__cmp__(other) != 0
 
-        def __lt__(self, other):
-            return self.__cmp__(other) < 0
+    def __gt__(self, other):
+        return self.__cmp__(other) > 0
 
-        def __ge__(self, other):
-            return self.__cmp__(other) >= 0
+    def __lt__(self, other):
+        return self.__cmp__(other) < 0
 
-        def __le__(self, other):
-            return self.__cmp__(other) <= 0
+    def __ge__(self, other):
+        return self.__cmp__(other) >= 0
+
+    def __le__(self, other):
+        return self.__cmp__(other) <= 0
+
+    @property
+    def feet(self):
+        return units.feet(kilometers=self.kilometers)
+
+    @property
+    def ft(self):
+        return self.feet
 
     @property
     def kilometers(self):
@@ -244,29 +412,21 @@ class Distance(object):
     def km(self):
         return self.kilometers
 
-    @property
-    def meters(self):
-        return units.meters(kilometers=self.kilometers)
-
     @property
     def m(self):
         return self.meters
 
     @property
-    def miles(self):
-        return units.miles(kilometers=self.kilometers)
+    def meters(self):
+        return units.meters(kilometers=self.kilometers)
 
     @property
     def mi(self):
         return self.miles
 
     @property
-    def feet(self):
-        return units.feet(kilometers=self.kilometers)
-
-    @property
-    def ft(self):
-        return self.feet
+    def miles(self):
+        return units.miles(kilometers=self.kilometers)
 
     @property
     def nautical(self):
@@ -279,7 +439,7 @@ class Distance(object):
 
 class great_circle(Distance):
     """
-    Use spherical geometry to calculate the surface distance between two
+    Use spherical geometry to calculate the surface distance between
     points.
 
     Set which radius of the earth to use by specifying a ``radius`` keyword
@@ -298,10 +458,11 @@ class great_circle(Distance):
 
     def __init__(self, *args, **kwargs):
         self.RADIUS = kwargs.pop('radius', EARTH_RADIUS)
-        super(great_circle, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def measure(self, a, b):
         a, b = Point(a), Point(b)
+        _ensure_same_altitude(a, b)
 
         lat1, lng1 = radians(degrees=a.latitude), radians(degrees=a.longitude)
         lat2, lng2 = radians(degrees=b.latitude), radians(degrees=b.longitude)
@@ -320,9 +481,6 @@ class great_circle(Distance):
         return self.RADIUS * d
 
     def destination(self, point, bearing, distance=None):
-        """
-        TODO docs.
-        """
         point = Point(point)
         lat1 = units.radians(degrees=point.latitude)
         lng1 = units.radians(degrees=point.longitude)
@@ -353,7 +511,7 @@ GreatCircleDistance = great_circle
 
 class geodesic(Distance):
     """
-    Calculate the geodesic distance between two points.
+    Calculate the geodesic distance between points.
 
     Set which ellipsoidal model of the earth to use by specifying an
     ``ellipsoid`` keyword argument. The default is 'WGS-84', which is the
@@ -371,28 +529,18 @@ class geodesic(Distance):
         >>> print(geodesic(newport_ri, cleveland_oh).miles)
         538.390445368
 
-
-    .. versionadded:: 1.13.0
     """
 
-    ellipsoid_key = None
-    ELLIPSOID = None
-    geod = None
-
     def __init__(self, *args, **kwargs):
+        self.ellipsoid_key = None
+        self.ELLIPSOID = None
+        self.geod = None
         self.set_ellipsoid(kwargs.pop('ellipsoid', 'WGS-84'))
-        if 'iterations' in kwargs:
-            warnings.warn('Ignoring unused `iterations` kwarg for geodesic '
-                          'distance.', DeprecationWarning, stacklevel=2)
-        kwargs.pop('iterations', 0)
         major, minor, f = self.ELLIPSOID
-        super(geodesic, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def set_ellipsoid(self, ellipsoid):
-        """
-        Change the ellipsoid used in the calculation.
-        """
-        if not isinstance(ellipsoid, (list, tuple)):
+        if isinstance(ellipsoid, str):
             try:
                 self.ELLIPSOID = ELLIPSOIDS[ellipsoid]
                 self.ellipsoid_key = ellipsoid
@@ -403,11 +551,10 @@ class geodesic(Distance):
         else:
             self.ELLIPSOID = ellipsoid
             self.ellipsoid_key = None
-        return
 
-    # Call geographiclib routines for measure and destination
     def measure(self, a, b):
         a, b = Point(a), Point(b)
+        _ensure_same_altitude(a, b)
         lat1, lon1 = a.latitude, a.longitude
         lat2, lon2 = b.latitude, b.longitude
 
@@ -422,9 +569,6 @@ class geodesic(Distance):
         return s12
 
     def destination(self, point, bearing, distance=None):
-        """
-        TODO docs.
-        """
         point = Point(point)
         lat1 = point.latitude
         lon1 = point.longitude
@@ -448,260 +592,5 @@ class geodesic(Distance):
 
 GeodesicDistance = geodesic
 
-
-class vincenty(Distance):
-    """
-    .. deprecated:: 1.13
-       Use :class:`.geodesic` instead.
-       Vincenty will be removed in geopy 2.0.
-
-    Calculate the geodesic distance between two points using the Vincenty's
-    method.
-
-    Set which ellipsoidal model of the earth to use by specifying an
-    ``ellipsoid`` keyword argument. The default is 'WGS-84', which is the
-    most globally accurate model.  If ``ellipsoid`` is a string, it is
-    looked up in the `ELLIPSOIDS` dictionary to obtain the major and minor
-    semiaxes and the flattening. Otherwise, it should be a tuple with those
-    values.  See the comments above the `ELLIPSOIDS` dictionary for
-    more information.
-
-    Example::
-
-        >>> from geopy.distance import vincenty
-        >>> newport_ri = (41.49008, -71.312796)
-        >>> cleveland_oh = (41.499498, -81.695391)
-        >>> print(vincenty(newport_ri, cleveland_oh).miles)
-        538.390445362
-
-    Note: Vincenty's method for distance fails to converge for some
-    valid (nearly antipodal) points. In such cases, use
-    :class:`.geodesic` which always produces an accurate result.
-
-    """
-
-    ellipsoid_key = None
-    ELLIPSOID = None
-    _show_deprecation_warning = True
-
-    def __init__(self, *args, **kwargs):
-        if self._show_deprecation_warning:
-            warnings.warn('Vincenty is deprecated and is going to be removed '
-                          'in geopy 2.0. Use `geopy.distance.geodesic` '
-                          '(or the default `geopy.distance.distance`) '
-                          'instead, which is more accurate and always converges.',
-                          DeprecationWarning, stacklevel=2)
-        self.set_ellipsoid(kwargs.pop('ellipsoid', 'WGS-84'))
-        self.iterations = kwargs.pop('iterations', 20)
-        major, minor, f = self.ELLIPSOID
-        super(vincenty, self).__init__(*args, **kwargs)
-
-    def set_ellipsoid(self, ellipsoid):
-        """
-        Change the ellipsoid used in the calculation.
-        """
-        if not isinstance(ellipsoid, (list, tuple)):
-            try:
-                self.ELLIPSOID = ELLIPSOIDS[ellipsoid]
-                self.ellipsoid_key = ellipsoid
-            except KeyError:
-                raise Exception(
-                    "Invalid ellipsoid. See geopy.distance.ELIPSOIDS"
-                )
-        else:
-            self.ELLIPSOID = ellipsoid
-            self.ellipsoid_key = None
-        return
-
-    def measure(self, a, b):
-        a, b = Point(a), Point(b)
-        lat1, lng1 = radians(degrees=a.latitude), radians(degrees=a.longitude)
-        lat2, lng2 = radians(degrees=b.latitude), radians(degrees=b.longitude)
-
-        if isinstance(self.ELLIPSOID, string_compare):
-            major, minor, f = ELLIPSOIDS[self.ELLIPSOID]
-        else:
-            major, minor, f = self.ELLIPSOID
-
-        delta_lng = lng2 - lng1
-
-        reduced_lat1 = atan((1 - f) * tan(lat1))
-        reduced_lat2 = atan((1 - f) * tan(lat2))
-
-        sin_reduced1, cos_reduced1 = sin(reduced_lat1), cos(reduced_lat1)
-        sin_reduced2, cos_reduced2 = sin(reduced_lat2), cos(reduced_lat2)
-
-        lambda_lng = delta_lng
-        lambda_prime = 2 * pi
-
-        iter_limit = self.iterations
-
-        i = 0
-
-        while (i == 0 or
-               (abs(lambda_lng - lambda_prime) > 10e-12 and i <= iter_limit)):
-            i += 1
-
-            sin_lambda_lng, cos_lambda_lng = sin(lambda_lng), cos(lambda_lng)
-
-            sin_sigma = sqrt(
-                (cos_reduced2 * sin_lambda_lng) ** 2 +
-                (cos_reduced1 * sin_reduced2 -
-                 sin_reduced1 * cos_reduced2 * cos_lambda_lng) ** 2
-            )
-
-            if sin_sigma == 0:
-                return 0  # Coincident points
-
-            cos_sigma = (
-                sin_reduced1 * sin_reduced2 +
-                cos_reduced1 * cos_reduced2 * cos_lambda_lng
-            )
-
-            sigma = atan2(sin_sigma, cos_sigma)
-
-            sin_alpha = (
-                cos_reduced1 * cos_reduced2 * sin_lambda_lng / sin_sigma
-            )
-            cos_sq_alpha = 1 - sin_alpha ** 2
-
-            if cos_sq_alpha != 0:
-                cos2_sigma_m = cos_sigma - 2 * (
-                    sin_reduced1 * sin_reduced2 / cos_sq_alpha
-                )
-            else:
-                cos2_sigma_m = 0.0  # Equatorial line
-
-            C = f / 16. * cos_sq_alpha * (4 + f * (4 - 3 * cos_sq_alpha))
-
-            lambda_prime = lambda_lng
-            lambda_lng = (
-                delta_lng + (1 - C) * f * sin_alpha * (
-                    sigma + C * sin_sigma * (
-                        cos2_sigma_m + C * cos_sigma * (
-                            -1 + 2 * cos2_sigma_m ** 2
-                        )
-                    )
-                )
-            )
-
-        if i > iter_limit:
-            raise ValueError("Vincenty formula failed to converge!")
-
-        u_sq = cos_sq_alpha * (major ** 2 - minor ** 2) / minor ** 2
-
-        A = 1 + u_sq / 16384. * (
-            4096 + u_sq * (-768 + u_sq * (320 - 175 * u_sq))
-        )
-
-        B = u_sq / 1024. * (256 + u_sq * (-128 + u_sq * (74 - 47 * u_sq)))
-
-        delta_sigma = (
-            B * sin_sigma * (
-                cos2_sigma_m + B / 4. * (
-                    cos_sigma * (
-                        -1 + 2 * cos2_sigma_m ** 2
-                    ) - B / 6. * cos2_sigma_m * (
-                        -3 + 4 * sin_sigma ** 2
-                    ) * (
-                        -3 + 4 * cos2_sigma_m ** 2
-                    )
-                )
-            )
-        )
-
-        s = minor * A * (sigma - delta_sigma)
-        return s
-
-    def destination(self, point, bearing, distance=None):
-        """
-        TODO docs.
-        """
-        point = Point(point)
-        lat1 = units.radians(degrees=point.latitude)
-        lng1 = units.radians(degrees=point.longitude)
-        bearing = units.radians(degrees=bearing)
-
-        if distance is None:
-            distance = self
-        if isinstance(distance, Distance):
-            distance = distance.kilometers
-
-        ellipsoid = self.ELLIPSOID
-        if isinstance(ellipsoid, string_compare):
-            ellipsoid = ELLIPSOIDS[ellipsoid]
-
-        major, minor, f = ellipsoid
-
-        tan_reduced1 = (1 - f) * tan(lat1)
-        cos_reduced1 = 1 / sqrt(1 + tan_reduced1 ** 2)
-        sin_reduced1 = tan_reduced1 * cos_reduced1
-        sin_bearing, cos_bearing = sin(bearing), cos(bearing)
-        sigma1 = atan2(tan_reduced1, cos_bearing)
-        sin_alpha = cos_reduced1 * sin_bearing
-        cos_sq_alpha = 1 - sin_alpha ** 2
-        u_sq = cos_sq_alpha * (major ** 2 - minor ** 2) / minor ** 2
-
-        A = 1 + u_sq / 16384. * (
-            4096 + u_sq * (-768 + u_sq * (320 - 175 * u_sq))
-        )
-        B = u_sq / 1024. * (256 + u_sq * (-128 + u_sq * (74 - 47 * u_sq)))
-
-        sigma = distance / (minor * A)
-        sigma_prime = 2 * pi
-
-        while abs(sigma - sigma_prime) > 10e-12:
-            cos2_sigma_m = cos(2 * sigma1 + sigma)
-            sin_sigma, cos_sigma = sin(sigma), cos(sigma)
-            delta_sigma = B * sin_sigma * (
-                cos2_sigma_m + B / 4. * (
-                    cos_sigma * (
-                        -1 + 2 * cos2_sigma_m ** 2
-                    ) - B / 6. * cos2_sigma_m * (
-                        -3 + 4 * sin_sigma ** 2
-                    ) * (
-                        -3 + 4 * cos2_sigma_m ** 2
-                    )
-                )
-            )
-            sigma_prime = sigma
-            sigma = distance / (minor * A) + delta_sigma
-
-        sin_sigma, cos_sigma = sin(sigma), cos(sigma)
-
-        lat2 = atan2(
-            sin_reduced1 * cos_sigma + cos_reduced1 * sin_sigma * cos_bearing,
-            (1 - f) * sqrt(
-                sin_alpha ** 2 + (
-                    sin_reduced1 * sin_sigma -
-                    cos_reduced1 * cos_sigma * cos_bearing
-                ) ** 2
-            )
-        )
-
-        lambda_lng = atan2(
-            sin_sigma * sin_bearing,
-            cos_reduced1 * cos_sigma - sin_reduced1 * sin_sigma * cos_bearing
-        )
-
-        C = f / 16. * cos_sq_alpha * (4 + f * (4 - 3 * cos_sq_alpha))
-
-        delta_lng = (
-            lambda_lng - (1 - C) * f * sin_alpha * (
-                sigma + C * sin_sigma * (
-                    cos2_sigma_m + C * cos_sigma * (
-                        -1 + 2 * cos2_sigma_m ** 2
-                    )
-                )
-            )
-        )
-
-        lng2 = lng1 + delta_lng
-
-        return Point(units.degrees(radians=lat2), units.degrees(radians=lng2))
-
-
-VincentyDistance = vincenty
-
 # Set the default distance formula
 distance = GeodesicDistance
