@@ -1,14 +1,8 @@
-import pytest
-
 from geopy.geocoders import AzureMaps
 from test.geocoders.tomtom import BaseTestTomTom
 from test.geocoders.util import env
 
 
-@pytest.mark.skipif(
-    not bool(env.get('AZURE_SUBSCRIPTION_KEY')),
-    reason="No AZURE_SUBSCRIPTION_KEY env variable set"
-)
 class TestAzureMaps(BaseTestTomTom):
 
     @classmethod
