@@ -1,5 +1,3 @@
-import pytest
-
 from geopy.geocoders import Geolake
 from test.geocoders.util import BaseTestGeocoder, env
 
@@ -14,10 +12,6 @@ class TestUnitGeolake:
         assert geocoder.headers['User-Agent'] == 'my_user_agent/1.0'
 
 
-@pytest.mark.skipif(
-    not bool(env.get('GEOLAKE_KEY')),
-    reason="No GEOLAKE_KEY env variables set"
-)
 class TestGeolake(BaseTestGeocoder):
 
     @classmethod
@@ -36,14 +30,24 @@ class TestGeolake(BaseTestGeocoder):
 
     async def test_geocode_country_codes_str(self):
         await self.geocode_run(
-            {"query": "Toronto", "country_codes": "US"},
-            {"latitude": 40.46, "longitude": -80.6, "address": "Toronto, US"},
+            {"query": "Toronto", "country_codes": "CA"},
+            {"latitude": 43.72, "longitude": -79.47, "address": "Toronto, CA"},
+        )
+        await self.geocode_run(
+            {"query": "Toronto", "country_codes": "RU"},
+            {},
+            expect_failure=True
         )
 
     async def test_geocode_country_codes_list(self):
         await self.geocode_run(
-            {"query": "Toronto", "country_codes": ["CN", "US"]},
-            {"latitude": 40.46, "longitude": -80.6, "address": "Toronto, US"},
+            {"query": "Toronto", "country_codes": ["CA", "RU"]},
+            {"latitude": 43.72, "longitude": -79.47, "address": "Toronto, CA"},
+        )
+        await self.geocode_run(
+            {"query": "Toronto", "country_codes": ["UA", "RU"]},
+            {},
+            expect_failure=True
         )
 
     async def test_geocode_structured(self):
