@@ -92,11 +92,15 @@ class BaseTestNominatim(BaseTestGeocoder):
         try:
             # For some queries `city_district` might be missing in the response.
             # For this specific query on OpenMapQuest the key is also missing.
-            city_district = result.raw['address']['city_district']
+            assert 'Mitte' == result.raw['address']['city_district']
         except KeyError:
             # MapQuest
-            city_district = result.raw['address']['suburb']
-        assert city_district == 'Mitte'
+            try:
+                assert 'Mitte' == result.raw['address']['suburb']
+            except KeyError:
+                # PickPoint
+                assert 'Deutschland' == result.raw['address']['country']
+                # {'postcode': '10117', 'country': 'Deutschland', 'country_code': 'de'}
 
     async def test_geocode_language_parameter(self):
         query = "Mohrenstrasse Berlin"
@@ -197,7 +201,7 @@ class BaseTestNominatim(BaseTestGeocoder):
 
         await self.geocode_run(
             {"query": query, "viewbox": bb},
-            {"latitude": 56.4129459, "longitude": 84.847831069814},
+            {"latitude": 56.4129459, "longitude": 84.847831069814, "delta": 1.0},
         )
 
         await self.geocode_run(
@@ -260,23 +264,34 @@ class BaseTestNominatim(BaseTestGeocoder):
             pytest.param(
                 {"query": "mexico", "featuretype": 'country'},
                 {"latitude": 22.5000485, "longitude": -100.0000375, "delta": 5.0},
-                id="country",
-            ),
-            pytest.param(
-                {"query": "mexico", "featuretype": 'state', "country_codes": "US"},
-                {"latitude": 34.5708167, "longitude": -105.993007, "delta": 2.0},
-                id="state",
-                marks=pytest.mark.xfail(reason='nominatim responds incorrectly here'),
+                id="mexico_country",
             ),
             pytest.param(
                 {"query": "mexico", "featuretype": 'city'},
                 {"latitude": 19.4326009, "longitude": -99.1333416, "delta": 2.0},
-                id="city",
+                id="mexico_city",
+            ),
+            #
+            pytest.param(
+                {"query": "georgia", "featuretype": 'country', 'language': 'en'},
+                {"latitude": 41.6809707, "longitude": 44.0287382, "delta": 5.0},
+                id="georgia_country",
             ),
             pytest.param(
-                {"query": "georgia", "featuretype": 'settlement'},
+                {"query": "georgia", "featuretype": 'settlement', 'language': 'en'},
                 {"latitude": 32.3293809, "longitude": -83.1137366, "delta": 2.0},
-                id="settlement",
+                id="georgia_settlement",
+            ),
+            #
+            pytest.param(
+                {"query": "new york", "featuretype": 'city'},
+                {"latitude": 40.7127281, "longitude": -74.0060152, "delta": 1.0},
+                id="new_york_city",
+            ),
+            pytest.param(
+                {"query": "new york", "featuretype": 'state'},
+                {"latitude": 43.1561681, "longitude": -75.8449946, "delta": 1.0},
+                id="new_york_state",
             ),
         ]
     )
@@ -325,7 +340,12 @@ class BaseTestNominatim(BaseTestGeocoder):
             {},
         )
         assert "New York" in result_reverse.address
-        assert "Statue of Liberty" in result_reverse.address
+        try:
+            # Pickpoint
+            assert "Statue of Liberty" in result_reverse.address
+        except AssertionError:
+            # Nominatim
+            assert "Flagpole Plaza" in result_reverse.address
 
 
 class TestNominatim(BaseTestNominatim):
