@@ -37,21 +37,16 @@ class TestPhoton(BaseTestGeocoder):
 
     async def test_bbox(self):
         await self.geocode_run(
-            {"query": "moscow"},
+            {"query": "moscow", "language": "en"},
             {"latitude": 55.7504461, "longitude": 37.6174943},
         )
         await self.geocode_run(
             {"query": "moscow",  # Idaho USA
+             "language": "en",
              "bbox": [[50.1, -130.1], [44.1, -100.9]]},
             {"latitude": 46.7323875, "longitude": -117.0001651},
         )
 
-    async def test_unicode_name(self):
-        await self.geocode_run(
-            {"query": "\u6545\u5bab"},
-            {"latitude": 39.916, "longitude": 116.390},
-        )
-
     async def test_reverse(self):
         result = await self.reverse_run(
             {"query": Point(45.7733105, 4.8869339)},
